/*------------------------------------------------------------------------------------
   Name: TDI-MTF-zz-Matrix.mq4
   
   Description: 3 Timeframe matrix indicator of the TDI-MTF-zz indicator written by zznbrm
   
   Requirements: The following indicators must installed and working: TDI-MTF-zz.mq4
                 http://www.forexfactory.com/showthread.php?p=4622169#post4622169  

   Change log:
       2014-03-24. Xaphod, v1.600
          - Update for build 600+
          - Removed indicator file check. 
       2014-01-09. Xaphod, v1.01
          - Fixed alerts
       2013-12-16. Xaphod, v1.00
          - Initial Release

-------------------------------------------------------------------------------------*/
// Indicator properties
#property copyright "Copyright 2013, Xaphod"
#property link      "http://www.xaphod.com"

//#property strict
#property version    "1.600"
#property description "3 Timeframe matrix indicator of the TDI-MTF-zz indicator written by zznbrm"
#property description " "
#property description "Requirements"
#property description "  The following indicators must installed and working:"
#property description "  TDI-MTF-zz.mq4"
#property description " "
#property description "Troubleshooting" 
#property description "  - Check the 'Journal' and 'Expert' tabs in the terminal window for errors."
#property description "  - Enable dlls to allow the indicator to check if the required files exist in the file system."

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1 DodgerBlue
#property indicator_color2 Red
#property indicator_color3 DodgerBlue
#property indicator_color4 Red
#property indicator_color5 DodgerBlue
#property indicator_color6 Red
#property indicator_minimum 0
#property indicator_maximum 1.6

//#include <xPrint.mqh>

// Constant definitions
#define INDICATOR_NAME "TDI-MTF-zz"
#define MATRIX_CHAR 110
#define IND_TDIMTFZZ "TDI-MTF-zz"


#define IDX_R1 0  // TF1
#define IDX_R2 1  // TF2
#define IDX_R3 2  // TF3
 
// Indicator parameters
extern string    TimeFrame_Settings=""; /* TimeFrame Settings */
extern int       TimeFrameAuto=2;     /* AutoSelect TimeFrame (0/1/2/3)*/ // Automatically select higher TF. M15 and M30 -> H1. Off=0, 1st HTF=1, 2nd HTF=2
extern string    TDI_Settings="";  /* TDI Indicator Settings */
extern int       TimeFrame1=15;        /* TimeFrame 1 Period */  // Timeframe: 0,1,5,15,30,60,240,1440 etc. Current Timeframe=0. 
extern int       RsiPeriod1=13;        /* TimeFrame 1 RSI Period */
extern int       PricePeriod1=2;       /* TimeFrame 1 Price Period */
extern int       SignalPeriod1=7;      /* TimeFrame 1 Signal Period */
extern int       TimeFrame2=60;        /* TimeFrame 2 Period */  // Timeframe: 0,1,5,15,30,60,240,1440 etc. Current Timeframe=0.
extern int       RsiPeriod2=13;        /* TimeFrame 2 RSI Period */
extern int       PricePeriod2=2;       /* TimeFrame 2 Price Period */
extern int       SignalPeriod2=7;      /* TimeFrame 2 Signal Period */
extern int       TimeFrame3=240;       /* TimeFrame 3 Period */  // Timeframe: 0,1,5,15,30,60,240,1440 etc. Current Timeframe=0.
extern int       RsiPeriod3=13;        /* TimeFrame 3 RSI Period */
extern int       PricePeriod3=2;       /* TimeFrame 3 Price Period */
extern int       SignalPeriod3=7;      /* TimeFrame 3 Signal Period */
extern int       MaxBars=2000;         /* Max Bars to Draw */
extern string    Alert_Settings="";   /* Alert Settings */
extern int       Alert_OnBar=1;        /* Alert On-Bar-Number */    // Alert on the change of this bar. 0=on open bar. 1=on closed bar 
extern string    Alert_Lines="111";    /* Alert Trigger Lines */    // matrix lines that must align for than alert to trigger
extern bool      Alert_Popup=True;     /* Alert Popup Enable */     // Enable popup window & sound on alert
extern string    Alert_Sound="";       /* Alert Sound Enable */     // Play sound on alert. Wav files only
extern bool      Alert_Email=False;    /* Alert eMail Enable */     // Enable send email on alert
extern string    Alert_Subject="";     /* Alert eMail Subject */    // Email Subject. Null string ("") will result in a preconfigured subject.
extern string    Label_Settings="";   /* Label Settings */
extern color     Label_Color=White;         // Color of Histogram Id labels


// Global module varables
// Histogram
double gadR1Up[];
double gadR1Dn[];
double gadR2Up[];
double gadR2Dn[];
double gadR3Up[];
double gadR3Dn[];

// Labels
double gadGap[4];

// Globals
int giAlert;
int giTimeFrame[3];
string gsIndicatorName;
bool gbInit;

//-----------------------------------------------------------------------------
// function: init()
// Description: Custom indicator initialization function.
//-----------------------------------------------------------------------------
int init() {
  // Init indicator buffers
  IndicatorBuffers(6);
  SetIndexStyle(0,DRAW_ARROW);
  SetIndexBuffer(0,gadR1Up);
  SetIndexLabel(0,NULL);
  SetIndexArrow(0,MATRIX_CHAR);
  SetIndexStyle(1,DRAW_ARROW);
  SetIndexBuffer(1,gadR1Dn);
  SetIndexLabel(1,NULL);
  SetIndexArrow(1,MATRIX_CHAR);
  SetIndexStyle(2,DRAW_ARROW);
  SetIndexBuffer(2,gadR2Up);
  SetIndexLabel(2,NULL);
  SetIndexArrow(2,MATRIX_CHAR);
  SetIndexStyle(3,DRAW_ARROW);
  SetIndexBuffer(3,gadR2Dn);  
  SetIndexLabel(3,NULL);
  SetIndexArrow(3,MATRIX_CHAR);
  SetIndexStyle(4,DRAW_ARROW);
  SetIndexBuffer(4,gadR3Up);
  SetIndexLabel(4,NULL);
  SetIndexArrow(4,MATRIX_CHAR);
  SetIndexStyle(5,DRAW_ARROW);
  SetIndexBuffer(5,gadR3Dn);
  SetIndexLabel(5,NULL);
  SetIndexArrow(5,MATRIX_CHAR);
  
  // Set Timeframe
  if (TimeFrameAuto==1) {
    giTimeFrame[IDX_R1]=Period();
    giTimeFrame[IDX_R2]=NextHigherTF(giTimeFrame[IDX_R1]);
    giTimeFrame[IDX_R3]=NextHigherTF(giTimeFrame[IDX_R2]); 
  }
  else if (TimeFrameAuto==2) {
    giTimeFrame[IDX_R1]=NextHigherTF(Period());
    giTimeFrame[IDX_R2]=NextHigherTF(giTimeFrame[IDX_R1]);
    giTimeFrame[IDX_R3]=NextHigherTF(giTimeFrame[IDX_R2]); 
  }
  else if (TimeFrameAuto==3) {
    giTimeFrame[IDX_R1]=NextHigherTF(NextHigherTF(Period()));
    giTimeFrame[IDX_R2]=NextHigherTF(giTimeFrame[IDX_R1]);
    giTimeFrame[IDX_R3]=NextHigherTF(giTimeFrame[IDX_R2]); 
  }
  else {
    giTimeFrame[IDX_R1]=TimeFrame1;
    giTimeFrame[IDX_R2]=TimeFrame2;
    giTimeFrame[IDX_R3]=TimeFrame3;
  } 
  
  // Set histogram positions
  gadGap[IDX_R1]=1.4;
  gadGap[IDX_R2]=0.9;
  gadGap[IDX_R3]=0.4;
  
  // Init alert
  giAlert=0;
  for (int i=0; i<3; i++) 
    if (StringSubstr(Alert_Lines,i,1)=="1")
      giAlert=giAlert+(int)MathPow(2,i);
  
  // Misc
  gsIndicatorName=INDICATOR_NAME+"-Matrix ("+TF2Str(giTimeFrame[IDX_R1])+"/"+TF2Str(giTimeFrame[IDX_R2])+"/"+TF2Str(giTimeFrame[IDX_R3])+")";
  IndicatorShortName(gsIndicatorName);
  gbInit=True;
  return(0);
}


//-----------------------------------------------------------------------------
// function: deinit()
// Description: Custom indicator deinitialization function.
//-----------------------------------------------------------------------------
int deinit() {
  // Clear text objects
  for(int i=ObjectsTotal()-1; i>-1; i--)
    if (StringFind(ObjectName(i),gsIndicatorName)>=0)  ObjectDelete(ObjectName(i));
  return (0);
}


//-----------------------------------------------------------------------------
// function: start()
// Description: Custom indicator iteration function.
//-----------------------------------------------------------------------------
int start() {
  int i, iNewBars, iCountedBars;
  double dSig0, dSig1;
  static datetime tCurBar=0;
  
  // Get unprocessed bars
  iCountedBars=IndicatorCounted();
  if(iCountedBars < 0) return (-1); 
  if(iCountedBars>0) iCountedBars--;
  iNewBars=Bars-iCountedBars;
  
  for(i=iNewBars; i>=0; i--) {
    if (i>=Bars)
      continue;
    // Clear buffers for parts not to be drawn
    if (i>MaxBars) {
      gadR1Up[i]=EMPTY_VALUE;
      gadR1Dn[i]=EMPTY_VALUE;
      gadR2Up[i]=EMPTY_VALUE;
      gadR2Dn[i]=EMPTY_VALUE;
      gadR3Up[i]=EMPTY_VALUE;
      gadR3Dn[i]=EMPTY_VALUE;
      continue;
    }
  
    // TDI for TF1
    dSig0=iCustom(NULL,0,IND_TDIMTFZZ,giTimeFrame[IDX_R1],RsiPeriod1,PricePeriod1,PRICE_CLOSE,SignalPeriod1,0,MaxBars,0,i);
    dSig1=iCustom(NULL,0,IND_TDIMTFZZ,giTimeFrame[IDX_R1],RsiPeriod1,PricePeriod1,PRICE_CLOSE,SignalPeriod1,0,MaxBars,1,i);
    if (dSig0>=dSig1) {
      gadR1Up[i]=gadGap[IDX_R1];
      gadR1Dn[i]=EMPTY_VALUE;
    }
    else {
      gadR1Up[i]=EMPTY_VALUE;
      gadR1Dn[i]=gadGap[IDX_R1];
    } 
    
    // TDI for TF2
    dSig0=iCustom(NULL,0,IND_TDIMTFZZ,giTimeFrame[IDX_R2],RsiPeriod2,PricePeriod2,PRICE_CLOSE,SignalPeriod2,0,MaxBars,0,i);
    dSig1=iCustom(NULL,0,IND_TDIMTFZZ,giTimeFrame[IDX_R2],RsiPeriod2,PricePeriod2,PRICE_CLOSE,SignalPeriod2,0,MaxBars,1,i);
    if (dSig0>=dSig1) {
      gadR2Up[i]=gadGap[IDX_R2];
      gadR2Dn[i]=EMPTY_VALUE;
    }
    else  {
      gadR2Up[i]=EMPTY_VALUE;
      gadR2Dn[i]=gadGap[IDX_R2];
    }  
    
    // TDI for TF3
    dSig0=iCustom(NULL,0,IND_TDIMTFZZ,giTimeFrame[IDX_R3],RsiPeriod3,PricePeriod3,PRICE_CLOSE,SignalPeriod3,0,MaxBars,0,i);
    dSig1=iCustom(NULL,0,IND_TDIMTFZZ,giTimeFrame[IDX_R3],RsiPeriod3,PricePeriod3,PRICE_CLOSE,SignalPeriod3,0,MaxBars,1,i);
    if (dSig0>=dSig1) {
      gadR3Up[i]=gadGap[IDX_R3];
      gadR3Dn[i]=EMPTY_VALUE;
    }
    else {
      gadR3Up[i]=EMPTY_VALUE;
      gadR3Dn[i]=gadGap[IDX_R3];
    } 
  }
  
  // Alerts
  CheckAlert(Alert_OnBar);
  
  // Tasks to execute on bar close
  if (tCurBar<Time[0]) {
    tCurBar=Time[0];
    
    // Write/Update bar labels
    Writelabel(TF2Str(giTimeFrame[IDX_R1]),gadGap[IDX_R1]+0.15);
    Writelabel(TF2Str(giTimeFrame[IDX_R2]),gadGap[IDX_R2]+0.15);
    Writelabel(TF2Str(giTimeFrame[IDX_R3]),gadGap[IDX_R3]+0.15);
  }                   
  
  // Clear the init flag   
  gbInit=False;
  
  return(0);
}


//-----------------------------------------------------------------------------
// function: CheckAlert()
// Description: Check for new alerts
//-----------------------------------------------------------------------------
void CheckAlert(int iBar) {
  static int iPrevAlert=0;
  int iCurSignal=0;
  
  if (Alert_Popup || Alert_Email || Alert_Sound!="") {
    // range check error
    if (iBar<0 || iBar>=Bars)
      return;
  
    // Get current signal
    if (UpSignal(iBar))
      iCurSignal=1;
    else if (DnSignal(iBar))
      iCurSignal=-1;
    else
      iCurSignal=0;
    
    // Show alert  
    if (iPrevAlert!=iCurSignal) {
      iPrevAlert=iCurSignal;    
      if (iCurSignal==1)
         AlertNow(Symbol()+", "+TF2Str(Period())+": TDI Matrix Buy Signal.");
      else if (iCurSignal==-1)
         AlertNow(Symbol()+", "+TF2Str(Period())+": TDI Matrix Sell Signal.");
    }
  }
  return;
}


//-----------------------------------------------------------------------------
// function: AlertNow()
// Description: Signal the popup and email alerts
//-----------------------------------------------------------------------------
void AlertNow(string sAlertMsg) {
  //PrintD(sAlertMsg);
  // No alerts on startup
  if (gbInit)
    return;
  //Popup Alert 
  if (Alert_Popup) 
    Alert(INDICATOR_NAME, ", ", sAlertMsg);
  if (Alert_Sound!="")
    PlaySound(Alert_Sound);
  //Email Alert
  if (Alert_Email) {
    if (Alert_Subject=="")
      SendMail( sAlertMsg, "MT4 Alert!\n"+INDICATOR_NAME+"\n" + TimeToStr(Time[0],TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);
    else 
      SendMail( Alert_Subject, "MT4 Alert!\n"+INDICATOR_NAME+"\n" + TimeToStr(Time[0],TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);          
  }
  
  return;
}


//-----------------------------------------------------------------------------
// function: UpSignal()
// Description: Return true if there is an up signal
//-----------------------------------------------------------------------------
bool UpSignal(int n) {
  int iAlertSig=0;
  if (gadR1Up[n]!=EMPTY_VALUE && (giAlert & 1)>0) iAlertSig=iAlertSig+1;
  if (gadR2Up[n]!=EMPTY_VALUE && (giAlert & 2)>0) iAlertSig=iAlertSig+2;
  if (gadR3Up[n]!=EMPTY_VALUE && (giAlert & 4)>0) iAlertSig=iAlertSig+4;
  if (iAlertSig==giAlert) 
    return(True);
  return(False);
}


//-----------------------------------------------------------------------------
// function: DnSignal()
// Description: eturn true if there is a down signal
//-----------------------------------------------------------------------------
bool DnSignal(int n) {
  int iAlertSig=0;
  if (gadR1Dn[n]!=EMPTY_VALUE && (giAlert & 1)>0) iAlertSig=iAlertSig+1;
  if (gadR2Dn[n]!=EMPTY_VALUE && (giAlert & 2)>0) iAlertSig=iAlertSig+2;
  if (gadR3Dn[n]!=EMPTY_VALUE && (giAlert & 4)>0) iAlertSig=iAlertSig+4;
  if (iAlertSig==giAlert)
    return(True);
  return(False);
}


//-----------------------------------------------------------------------------
// function: Writelabel()
// Description: Write a label for a bar
//-----------------------------------------------------------------------------
int Writelabel(string sLabel,double dPrice) {
  string sObjId;
  sObjId=gsIndicatorName+"_"+sLabel;
  if(ObjectFind(sObjId) < 0) 
    ObjectCreate(sObjId, OBJ_TEXT, WindowFind(gsIndicatorName), Time[0]+Period()*60*2, dPrice);
  ObjectSetText(sObjId, sLabel, 6, "Lucida Console", Label_Color);
  ObjectMove(sObjId,0,Time[0]+Period()*60*2, dPrice);
  return(0);
}


//-----------------------------------------------------------------------------
// function: TF2Str()
// Description: Convert time-frame to a string
//-----------------------------------------------------------------------------
string TF2Str(int iPeriod) {
  switch(iPeriod) {
    case PERIOD_M1: return("M1");
    case PERIOD_M5: return("M5");
    case PERIOD_M15: return("M15");
    case PERIOD_M30: return("M30");
    case PERIOD_H1: return("H1");
    case PERIOD_H4: return("H4");
    case PERIOD_D1: return("D1");
    case PERIOD_W1: return("W1");
    case PERIOD_MN1: return("MN1");
    default: return("M"+(string)iPeriod);
  }
  return("M?");
}


//-----------------------------------------------------------------------------
// function: NextHigherTF()
// Description: Select the next higher time-frame. 
//              Note: M15 and M30 both select H1 as next higher TF. 
//-----------------------------------------------------------------------------
int NextHigherTF(int iPeriod) {
  if (iPeriod==0) iPeriod=Period();
  switch(iPeriod) {
    case PERIOD_M1: return(PERIOD_M5);
    case PERIOD_M5: return(PERIOD_M15);
    case PERIOD_M15: return(PERIOD_H1);
    case PERIOD_M30: return(PERIOD_H1);
    case PERIOD_H1: return(PERIOD_H4);
    case PERIOD_H4: return(PERIOD_D1);
    case PERIOD_D1: return(PERIOD_W1);
    case PERIOD_W1: return(PERIOD_MN1);
    case PERIOD_MN1: return(PERIOD_MN1);
    
    default: return(Period());
  }
  return(Period());
}






