

//+------------------------------------------------------------------+
//|  TradersDynamicIndex               Traders Dynamic Index.mq4     |
//|                                    Copyright  2006, Dean Malone |
//|                                    www.compassfx.com             |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|                                                                  |
//|                     Traders Dynamic Index                        |
//|                                                                  |
//|  This hybrid indicator is developed to assist traders in their   |
//|  ability to decipher and monitor market conditions related to    |
//|  trend direction, market strength, and market volatility.        |
//|                                                                  | 
//|  Even though comprehensive, the T.D.I. is easy to read and use.  |
//|                                                                  |
//|  Green line = RSI Price line                                     |
//|  Red line = Trade Signal line                                    |
//|  Blue lines = Volatility Band                                    | 
//|  Yellow line = Market Base Line                                  |  
//|                                                                  |
//|  Trend Direction - Immediate and Overall                         |
//|   Immediate = Green over Red...price action is moving up.        |
//|               Red over Green...price action is moving down.      |
//|                                                                  |   
//|   Overall = Yellow line trends up and down generally between the |
//|             lines 32 & 68. Watch for Yellow line to bounces off  |
//|             these lines for market reversal. Trade long when     |
//|             price is above the Yellow line, and trade short when |
//|             price is below.                                      |        
//|                                                                  |
//|  Market Strength & Volatility - Immediate and Overall            |
//|   Immediate = Green Line - Strong = Steep slope up or down.      | 
//|                            Weak = Moderate to Flat slope.        |
//|                                                                  |               
//|   Overall = Blue Lines - When expanding, market is strong and    |
//|             trending. When constricting, market is weak and      |
//|             in a range. When the Blue lines are extremely tight  |                                                       
//|             in a narrow range, expect an economic announcement   | 
//|             or other market condition to spike the market.       |
//|                                                                  |               
//|                                                                  |
//|  Entry conditions                                                |
//|   Scalping  - Long = Green over Red, Short = Red over Green      |
//|   Active - Long = Green over Red & Yellow lines                  |
//|            Short = Red over Green & Yellow lines                 |    
//|   Moderate - Long = Green over Red, Yellow, & 50 lines           |
//|              Short= Red over Green, Green below Yellow & 50 line |
//|                                                                  |
//|  Exit conditions*                                                |   
//|   Long = Green crosses below Red                                 |
//|   Short = Green crosses above Red                                |
//|   * If Green crosses either Blue lines, consider exiting when    |
//|     when the Green line crosses back over the Blue line.         |
//|                                                                  |
//|                                                                  |
//|  IMPORTANT: The default settings are well tested and proven.     |
//|             But, you can change the settings to fit your         |
//|             trading style.                                       |
//|                                                                  |
//|                                                                  |
//|  Price & Line Type settings:                           |                
//|   RSI Price settings                                             |               
//|   0 = Close price     [DEFAULT]                                  |               
//|   1 = Open price.                                                |               
//|   2 = High price.                                                |               
//|   3 = Low price.                                                 |               
//|   4 = Median price, (high+low)/2.                                |               
//|   5 = Typical price, (high+low+close)/3.                         |               
//|   6 = Weighted close price, (high+low+close+close)/4.            |               
//|                                                                  |               
//|   RSI Price Line & Signal Line Type settings                                   |               
//|   0 = Simple moving average       [DEFAULT]                      |               
//|   1 = Exponential moving average                                 |               
//|   2 = Smoothed moving average                                    |               
//|   3 = Linear weighted moving average                             |               
//|                                                                  |
//|   Good trading,                                                  |   
//|                                                                  |
//|   Dean                                                           |                              
//+------------------------------------------------------------------+

// Trade Annoucements added to help focus attention on decisive events. 
//  John Burch,  www.forex-tools-cafe.com 
//  
//  Sounds off for various combinations of 
//  Going UP   when red > green, red > yellow , red > 50 
//  Going Down when red < green, red < yellow , red < 50
// 

#property indicator_buffers 6
#property indicator_color1 Black
#property indicator_color2 MediumBlue
#property indicator_color3 Yellow
#property indicator_color4 MediumBlue
#property indicator_color5 Green
#property indicator_color6 Red
#property indicator_separate_window

extern int RSI_Period = 13;         //8-25
extern int RSI_Price = 0;           //0-6
extern int Volatility_Band = 34;    //20-40
extern int RSI_Price_Line = 2;      
extern int RSI_Price_Type = 0;      //0-3
extern int Trade_Signal_Line = 7;   
extern int Trade_Signal_Type = 0;   //0-3


// added by Burch
extern string S1="*** Announcements";
extern bool   EnableAlert=true;
extern bool   EnableSound=true;
extern bool   EnableDefaultSound=false; // set true to use without a set of spoken audio files. 
extern string DefaultSoundFile="alert.wav";

bool          GreenRedFired=false;   // controls the sounds to once per transition
bool          GreenAboveRed=false; 
extern string S2="*** Type: 1=scalp, 2=active, 3=moderate, 4=PeakFinder, 5=modof4";
extern int    AnnounceType=5;       // 1=scalp (one condition, fastest) , 2= active ( two conditions), 3 = moderate ( 3 conditions, most delayed and safer )


double RSIBuf[],UpZone[],MdZone[],DnZone[],MaBuf[],MbBuf[];

int init()
{
    IndicatorShortName("TDI- TradeAnnoucer");
    SetIndexBuffer(0,RSIBuf);
    SetIndexBuffer(1,UpZone);
    SetIndexBuffer(2,MdZone);    // yellow
    SetIndexBuffer(3,DnZone);    // 
    SetIndexBuffer(4,MaBuf);     // green
    SetIndexBuffer(5,MbBuf);     // red

    SetIndexStyle(0,DRAW_NONE); 
    SetIndexStyle(1,DRAW_LINE); 
    SetIndexStyle(2,DRAW_LINE,0,2);
    SetIndexStyle(3,DRAW_LINE);
    SetIndexStyle(4,DRAW_LINE,0,2);
    SetIndexStyle(5,DRAW_LINE,0,2);

    SetIndexLabel(0,NULL); 
    SetIndexLabel(1,"VB High"); 
    SetIndexLabel(2,"Market Base Line"); 
    SetIndexLabel(3,"VB Low"); 
    SetIndexLabel(4,"RSI Price Line");
    SetIndexLabel(5,"Trade Signal Line");

    SetLevelValue(0,50);
    SetLevelValue(1,68);
    SetLevelValue(2,32);
    SetLevelStyle(STYLE_DOT,1,DimGray);

    return(0);
}



int start()
{
   double MA,RSI[];
   ArrayResize(RSI,Volatility_Band);
   int counted_bars=IndicatorCounted();
   int limit = Bars-counted_bars-1;
   
   for(int i=limit; i>=0; i--)
   {
        RSIBuf[i] = (iRSI(NULL,0,RSI_Period,RSI_Price,i)); 
        MA = 0;
        for(int x=i; x<i+Volatility_Band; x++) 
        {
            RSI[x-i] = RSIBuf[x];
            MA += RSIBuf[x]/Volatility_Band;
        }
        UpZone[i] = (MA + (1.6185 * StDev(RSI,Volatility_Band)));
        DnZone[i] = (MA - (1.6185 * StDev(RSI,Volatility_Band)));  
        MdZone[i] = ((UpZone[i] + DnZone[i])/2);
    }
    
    for (i=limit-1;i>=0;i--)  
    {
        MaBuf[i] = (iMAOnArray(RSIBuf,0,RSI_Price_Line,0,RSI_Price_Type,i));
        MbBuf[i] = (iMAOnArray(RSIBuf,0,Trade_Signal_Line,0,Trade_Signal_Type,i));   
    } 

    datetime t;
    bool     sayit = false;
    
    if ( !GreenRedFired && NewBar() )
    {
        if ( AnnounceType == 1 &&  MaBuf[0] >= MbBuf[0] && MaBuf[1] <= MbBuf[1]) 
            sayit = true; 
        else if ( AnnounceType == 2 &&  MaBuf[0] >= MbBuf[0] && MaBuf[1] <= MbBuf[1] &&  MaBuf[0] > MdZone[0] ) 
            sayit = true;
        else if ( AnnounceType == 3 &&  MaBuf[0] >= MbBuf[0] && MaBuf[1] <= MbBuf[1] &&  MaBuf[0] > MdZone[0] && MaBuf[0] >= 50) 
            sayit = true;
        else if ( AnnounceType == 4 &&  MaBuf[0] >= MbBuf[0] && MaBuf[1] <= MbBuf[1] &&  MdZone[0] <= 47.0 && ((MaBuf[0]+MbBuf[0])/2)< 45.0 ) 
            sayit = true;
        else if ( AnnounceType == 5 &&  MaBuf[0] >= MbBuf[0] && MaBuf[1] <= MbBuf[1] &&  ((MaBuf[0]+MbBuf[0])/2)< MdZone[0] ) 
            sayit = true;
             
        if ( !EnableDefaultSound )
        {
            if ( sayit && EnableSound )
            {
                 PlaySound("GOINGUP.wav");
             
                 t = TimeLocal(); 
                 while( TimeLocal() < t + 2 ) 
                     t = t ;
                 PlaySound(SymbolToSound() );
             
                 t = TimeLocal(); 
                 while( TimeLocal() < t + 2 ) 
                     t = t;
                 PlaySound(PeriodToSound() ); 
             
            }
        }
        else
            PlaySound(DefaultSoundFile);
            
        t = TimeLocal(); 
        while( TimeLocal() < t + 1 ) 
            t = t;  // so the alert sound does not step on the PlaySound
                 
        if ( sayit && EnableAlert ) Alert("TDI: "+Symbol()+" BUY Signal in " + GetPeriodName() );
        
        if ( sayit ) GreenRedFired= true; 
        
        sayit = false; 
        
        if ( AnnounceType == 1 &&  MaBuf[0] <= MbBuf[0] && MaBuf[1] >= MbBuf[1]) 
            sayit = true; 
        else if ( AnnounceType == 2 &&  MaBuf[0] <= MbBuf[0] && MaBuf[1] >= MbBuf[1] &&  MaBuf[0] < MdZone[0] ) 
            sayit = true;
        else if ( AnnounceType == 3 &&  MaBuf[0] <= MbBuf[0] && MaBuf[1] >= MbBuf[1] &&  MaBuf[0] < MdZone[0] && MaBuf[0] <= 50) 
            sayit = true;
         else if ( AnnounceType == 4 &&  MaBuf[0] <= MbBuf[0] && MaBuf[1] >= MbBuf[1] &&  MdZone[0] >= 53.0 && ((MaBuf[0]+MbBuf[0])/2) >= 55.0 ) 
            sayit = true;
         else if ( AnnounceType == 5 &&  MaBuf[0] <= MbBuf[0] && MaBuf[1] >= MbBuf[1] &&  ((MaBuf[0]+MbBuf[0])/2) > MdZone[0] ) 
            sayit = true;  
             
        if ( !EnableDefaultSound )
        {
        
            if ( sayit && EnableSound )
            {
                 PlaySound("GOINGDOWN.wav");
             
                 t = TimeLocal(); 
                 while( TimeLocal() < t + 2 ) 
                     t = t ;
                 PlaySound(SymbolToSound() );
             
                 t = TimeLocal(); 
                 while( TimeLocal() < t + 2 ) 
                     t = t;
                 PlaySound(PeriodToSound() ); 
            }
        }
        else
            PlaySound(DefaultSoundFile);
            
    
    
        t = TimeLocal(); 
        while( TimeLocal() < t + 1 ) 
            t = t;  // so the alert sound does not step on the PlaySound
                 
        if ( sayit && EnableAlert ) Alert("TDI: "+ Symbol() + " SELL Signal in " + GetPeriodName() );
        
        if ( sayit ) GreenRedFired= true; 

        
    }
    
    if ( GreenRedFired )
    {
        if (    GreenAboveRed )
            if ( MaBuf[0] < MbBuf[0]  )
            {
                GreenAboveRed=false;
                GreenRedFired=false;
            }
        else 
            if ( MaBuf[0] > MbBuf[0]  )
            {
                GreenAboveRed=true;
                GreenRedFired=false;
            }
    }
    
    return(0);
}



double StDev(double& Data[], int Per)
{
    return(MathSqrt(Variance(Data,Per)));
}



double Variance(double& Data[], int Per)
{
    double sum, ssum;
    for (int i=0; i<Per; i++)
    {
        sum += Data[i];
        ssum += MathPow(Data[i],2);
    }
    return((ssum*Per - sum*sum)/(Per*(Per-1)));
}



bool NewBar()
{
   static datetime lastbar = 0;
   datetime curbar = Time[0];
   if(lastbar!=curbar)
   {
      lastbar=curbar;
      return (true);
   }
   else
   {
      return(false);
   }
}  


string GetPeriodName()
{

    int peri = Period();
    string perstr; 
    switch(peri) {
        case PERIOD_M1: { perstr="M1"  ; break;}
        case PERIOD_M5: { perstr="M5"  ; break;}
        case PERIOD_M15: {perstr="M15" ; break;}
        case PERIOD_M30: {perstr="M30" ; break;}
        case PERIOD_H1:  {perstr="H1"  ; break;}
        case PERIOD_H4:  {perstr="H4"  ; break;}
        case PERIOD_D1:  {perstr="D1"  ; break;}
        case PERIOD_W1:  {perstr="W1"  ; break;}
        case PERIOD_MN1: {perstr="MN1";}
     } 
     return (perstr);
}

string SymbolToSound()
{

    string  pairs[]={"EURUSD","GBPUSD","AUDUSD","USDCHF","USDJPY","EURAUD","GBPCHF","GBPJPY","EURJPY","USDCAD"};

    string sound;
    int  k; 
    string  sym = Symbol();
    for ( int i=0; i<ArraySize(pairs); i++ )
    {
        if ( pairs[i] ==  sym )
            k= i; 
    }
    switch(k) {
       case 0:  sound="EUR_USD.wav"  ; break;
       case 1:  sound="GBP_USD.wav"  ; break;
       case 2: sound="AUD_USD.wav" ; break;
       case 3: sound="USD_CHF.wav" ; break;
       case 4:  sound="USD_JPY.wav"  ; break;
       case 5:  sound="EUR_AUD.wav"  ; break;
       case 6:  sound="GBP_CHF.wav"  ; break;
       case 7:  sound="GBP_JPY.wav"  ; break;
       case 8: sound="EUR_JPY.wav";break;
       case 9: sound="USD_CAD.wav";break;
       default:
            sound = "alert.wav"; 
    }
    return(sound);
}

string PeriodToSound()
{
   string sound;
   switch(Period()) {
      case PERIOD_M1:  sound="M1.wav"  ; break;
      case PERIOD_M5:  sound="M5.wav"  ; break;
      case PERIOD_M15: sound="M15.wav" ; break;
      case PERIOD_M30: sound="M30.wav" ; break;
      case PERIOD_H1:  sound="H1.wav"  ; break;
      case PERIOD_H4:  sound="H4.wav"  ; break;
      case PERIOD_D1:  sound="DAILY.wav"  ; break;
      case PERIOD_W1:  sound="WEEKLY.wav"  ; break;
      case PERIOD_MN1: sound="MONTHLY.wav";
   }
   return(sound);
}