/*------------------------------------------------------------------------------------
   Name: xTDI Strong-Histo.mq4
   Copyright 2010, Xaphod, http://forexwhiz.appspot.com
   
   Description: Displays histogram indicating 
                RSIPriceLine above TradeSignalLine, MarketBaseLine and 50 level and 
                RSIPriceLine below TradeSignalLine, MarketBaseLine and 50 level
                with optional alerts
                NOTE! Requires Traders_Dynamic_Index.mq4 is installed

   Change log: 
       2011-01-29. Xaphod, v1.00 
          - First Release
       2011-08-11. Xaphod, v1.01 
          - Update current bar on every tick 
-------------------------------------------------------------------------------------*/
// Indicator properties
#property copyright "Copyright  2010, Xaphod"
#property link      "http://forexwhiz.appspot.com"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  Red
#property indicator_color3  Gold
#property indicator_width1  4
#property indicator_width2  4
#property indicator_width3  4
#property indicator_maximum 1
#property indicator_minimum 0

// Constant definitions
#define INDICATOR_NAME "xTDI Strong-Histo"
#define INDICATOR_VERSION "v1.01"
#define BAR_HEIGHT 1.01
#define TDI_INDI "TDI_RT"

#define ALARM_OFF 0
#define ALARM_ON 1
#define ALARM_RESET 2


// Indicator parameters
extern string    Indi.Copyright= "2010, mindoverforex.com";
extern string    Indi.Version= INDICATOR_VERSION;
extern string    TDI.Settings="";
extern int       TDI.RSIPeriod = 13;           // Period. Recomended values: 8-25
extern int       TDI.RSIPrice = 0;             // 0=CLOSE, 1=OPEN, 2=HIGH, 3=LOW, 4=MEDIAN, 5=TYPICAL, 6=WEIGHTED
extern int       TDI.VolatilityBand = 34;      // Recomended values: 20-40
extern int       TDI.RSIPriceLine = 2;         // Period
extern int       TDI.RSIPriceType = 0;         // 0=SMA, 1=EMA, 2=SSMA, 3=LWMA
extern int       TDI.TradeSignalLine = 7;      // Period
extern int       TDI.TradeSignalType = 0;      // 0=SMA, 1=EMA, 2=SSMA, 3=LWMA
extern string    Alert.Settings="";
extern bool      Alert.Popup=true;             // Enable Popup dialog and sound alert
extern bool      Alert.Email=false;            // Enable EMail alert
extern bool      Alert.NewBarOnly=true;       // Only alert after current bar closes
extern int       Alert.Timeout=20;             // Temporarily disable alerts after alert on currently open bar


// Global module varables
double dLongSig[];
double dShortSig[];
double dNoSig[];
bool bStartup;

int       TDI.TimeFrame=0;
//-----------------------------------------------------------------------------
// function: init()
// Description: Custom indicator initialization function.
//-----------------------------------------------------------------------------
int init() {
   string sType;   
   string sTF;
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(0,dLongSig);
   SetIndexBuffer(1,dShortSig);
   SetIndexBuffer(2,dNoSig);
   SetIndexLabel(0,"Long");
   SetIndexLabel(1,"Short");
   SetIndexLabel(2,"None");
   
   //for (int i = 0; i < indicator_buffers; i++) SetIndexLabel(i,NULL);
    
  if (TDI.TimeFrame==0) 
    sTF="";
  else
    sTF=TF2Str(TDI.TimeFrame);
  IndicatorShortName(INDICATOR_NAME+" "+sTF);
  IndicatorDigits(2);

  bStartup=true;
  return(0);
}

//-----------------------------------------------------------------------------
// function: deinit()
// Description: Custom indicator deinitialization function.
//-----------------------------------------------------------------------------
int deinit() {
   return (0);
}


///-----------------------------------------------------------------------------
// function: start()
// Description: Custom indicator iteration function.
//-----------------------------------------------------------------------------
int start() {
  int iNewBars;
  int iCountedBars; 
  int i;
  double dVBHighLine;        // Upper volatility band - Upper Blue line
  double dVBLowLine;         // Lower volatility band - Lower Blue line
  double dMarketBaseLine;    // Market Base Line - Yellow line
  double dRSIPriceLine;      // RSI PriceLine - Green line
  double dTradeSignalLine;   // Trade Signal Line - Red line
  
  // Get unprocessed ticks
  iCountedBars=IndicatorCounted();
  if(iCountedBars < 0) return (-1); 
  if(iCountedBars>0) iCountedBars--;
  iNewBars=Bars-iCountedBars;
  
  for(i=iNewBars; i>=0; i--) {      
    // Get TDI Data
    dMarketBaseLine=iCustom(NULL,TDI.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                            TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,2,i);   // Yellow line
    dRSIPriceLine=iCustom(NULL,TDI.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                          TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,4,i);     // Green line
    dTradeSignalLine=iCustom(NULL,TDI.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                            TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,5,i);  // Red line
    
    dLongSig[i] = EMPTY_VALUE; 
    dShortSig[i] = EMPTY_VALUE;
    dNoSig[i] = EMPTY_VALUE;
    
    if (dRSIPriceLine>dTradeSignalLine && dRSIPriceLine>dMarketBaseLine && dRSIPriceLine>50)
        dLongSig[i] = BAR_HEIGHT;
    else if (dRSIPriceLine<dTradeSignalLine && dRSIPriceLine<dMarketBaseLine && dRSIPriceLine<50)
        dShortSig[i] = BAR_HEIGHT;
    else
      dNoSig[i] = BAR_HEIGHT;     

    // Check for alerts
    if (i==0) AlertCheck();
  }
  
  // Reset startup flag
  if (bStartup) bStartup=false;
  
  // Bye bye  
  return(0);
}
//+------------------------------------------------------------------+
 //if (i<6) PrintD("YE: i="+i+"  Gr="+DoubleToStr(dRSIPriceLine,Digits)+"  Ye="+DoubleToStr(dMarketBaseLine,Digits)+"  Re="+DoubleToStr(dTradeSignalLine,Digits));

//-----------------------------------------------------------------------------
// function: TF2Str()
// Description: Convert time-frame to a string
//-----------------------------------------------------------------------------
string TF2Str(int iPeriod) {
  switch(iPeriod) {
    case PERIOD_M1: return("M1");
    case PERIOD_M5: return("M5");
    case PERIOD_M15: return("M15");
    case PERIOD_M30: return("M30");
    case PERIOD_H1: return("H1");
    case PERIOD_H4: return("H4");
    case PERIOD_D1: return("D1");
    case PERIOD_W1: return("W1");
    case PERIOD_MN1: return("MN1");
    default: return("M"+iPeriod);
  }
}

//-----------------------------------------------------------------------------
// function: AlertCheck()
// Description: Alert if conditions are set
//-----------------------------------------------------------------------------
void AlertCheck() {
  string sAlertMsg;
  static datetime tCurTime=0;  
  static int iAlertTimer=0;
  
  // Abort if indicator just started up
  if (bStartup) {
    tCurTime=Time[0];
    return(0);
  }
  
  // Alert if newbar
  if (Alert.NewBarOnly) {
    // Reset Time
    if(tCurTime!=Time[0]) {
      tCurTime=Time[0];
    
      // Check cross above TSL & MBL & 50
      if (dLongSig[1]==BAR_HEIGHT && dLongSig[2]!=BAR_HEIGHT) {
        sAlertMsg="TDI Alert - "+Symbol()+" "+TF2Str(Period())+": RSIPriceLine Above TradeSignalLine, MarketBaseLine and 50 level.";
        if (Alert.Popup) 
          Alert(sAlertMsg);      
        if (Alert.Email) 
          SendMail( sAlertMsg, "MT4 Alert!\n" + TimeToStr(TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);
      }
      // Check cross above TSL & MBL & 50
      else if (dShortSig[1]==BAR_HEIGHT && dShortSig[2]!=BAR_HEIGHT) {
        sAlertMsg="TDI Alert - "+Symbol()+" "+TF2Str(Period())+": RSIPriceLine Below TradeSignalLine, MarketBaseLine and 50 level.";
        if (Alert.Popup) 
          Alert(sAlertMsg);      
        if (Alert.Email) 
          SendMail( sAlertMsg, "MT4 Alert!\n" + TimeToStr(TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);
      }
    }
  }
  // Alert on current bar
  else {
    if (iAlertTimer<GetTickCount()) {
      // Check cross above TSL & MBL & 50
      if (dLongSig[0]==BAR_HEIGHT && dLongSig[1]!=BAR_HEIGHT) {
        iAlertTimer=GetTickCount()+Alert.Timeout*1000;
        sAlertMsg="TDI Alert - "+Symbol()+" "+TF2Str(Period())+": RSIPriceLine Above TradeSignalLine, MarketBaseLine and 50 level.";
        if (Alert.Popup) 
          Alert(sAlertMsg);      
        if (Alert.Email) 
          SendMail( sAlertMsg, "MT4 Alert!\n" + TimeToStr(TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);
      }
      // Check cross above TSL & MBL & 50
      else if (dShortSig[1]==BAR_HEIGHT && dShortSig[2]!=BAR_HEIGHT) {
        iAlertTimer=GetTickCount()+Alert.Timeout*1000;
        sAlertMsg="TDI Alert - "+Symbol()+" "+TF2Str(Period())+": RSIPriceLine Below TradeSignalLine, MarketBaseLine and 50 level.";
        if (Alert.Popup) 
          Alert(sAlertMsg);      
        if (Alert.Email) 
          SendMail( sAlertMsg, "MT4 Alert!\n" + TimeToStr(TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);
      }
    }
  }
  return(0);
}

