/*------------------------------------------------------------------------------------
   Name: xTDI-Histo.mq4
   Copyright 2010, Xaphod, http://forexwhiz.appspot.com
   
   Description: - Draws a tape chart with data for the RSI price line (green), the
                  trade signal line (red) and the market base line (yellow.
                - Provides alerts for crossovers of the above three lines.
	          
   Change log: 
       2010-10-19. Xaphod, v1.00 
          - Initial Release 
       2010-10-22. Xaphod, v1.10 
          - Added Arrows   
-------------------------------------------------------------------------------------*/
// Indicator properties
#property copyright "Copyright  2010, Xaphod"
#property link      "http://forexwhiz.appspot.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  LimeGreen
#property indicator_color2  Red
#property indicator_color3  Green
#property indicator_color4  FireBrick
#property indicator_width1  4
#property indicator_width2  4
#property indicator_width3  4
#property indicator_width4  4
#property indicator_maximum 1
#property indicator_minimum 0

// Constant definitions
#define INDICATOR_NAME "xTDI-Histo"
#define INDICATOR_VERSION "v1.00"
#define BAR_HEIGHT 1.10
#define TDI_INDI "Traders_Dynamic_Index"

#define ARROW_THIN_UP 233
#define ARROW_THIN_DN 234
#define ARROW_THICK_UP 225
#define ARROW_THICK_DN 226
#define ARROW_HOLLOW_UP 241
#define ARROW_HOLLOW_DN 242


// Indicator parameters
extern string    Indi.Copyright= "2010, forexwhiz.appspot.com";
extern string    Indi.Version= INDICATOR_VERSION;
extern string    Show.Settings="";
extern int       Show.TimeFrame=0;             // Timeframe to use. Current Timeframe=0.
extern bool      Show.MarketBaseLine=true;     // Show cross above/below market baseline in a different color  
extern bool      Show.RSIPriceLineArrow=false;  // Show arrows at RSIPriceLineArrow/TradeSignalLine crosses on the chart
extern bool      Show.MarketBaseLineArrow=false;// Show arrows at RSIPriceLineArrow/MarketBaseLine crosses on the chart
extern int       Show.ArrowType=0;             // 0=Thick, 1=Thin, 2=Hollow, 3=Round
extern string    TDI.Settings="";
extern int       TDI.RSIPeriod = 13;           // Period. Recomended values: 8-25
extern int       TDI.RSIPrice = 0;             // 0=CLOSE, 1=OPEN, 2=HIGH, 3=LOW, 4=MEDIAN, 5=TYPICAL, 6=WEIGHTED
extern int       TDI.VolatilityBand = 34;      // Recomended values: 20-40
extern int       TDI.RSIPriceLine = 2;         // Period
extern int       TDI.RSIPriceType = 0;         // 0=SMA, 1=EMA, 2=SSMA, 3=LWMA
extern int       TDI.TradeSignalLine = 7;      // Period
extern int       TDI.TradeSignalType = 0;      // 0=SMA, 1=EMA, 2=SSMA, 3=LWMA
extern string    Alert.Settings="";
extern bool      Alert.RSIPriceLine=false;     // Alert if RSI price crosses Signal line. Green crosses red.
extern bool      Alert.MarketBaseLine=false;   // Alert if RSI price crosses Signal line & market base line. Green crosses red and yellow
extern bool      Alert.NewBarOnly=true;        // True=Alert when new bar is opened. False=Alert anytime lines cross
extern int       Alert.Timeout=60;             // Deactivate alerts for N seconds after an alert. Only when Alert.NewBarOnly=false
extern bool      Alert.Popup=true;             // Popup window & sound on alert
extern bool      Alert.Email=false;            // Send email on alert


// Global module varables
double dBuffer0[];
double dBuffer1[];
double dBuffer2[];
double dBuffer3[];
bool bStartup;


//-----------------------------------------------------------------------------
// function: init()
// Description: Custom indicator initialization function.
//-----------------------------------------------------------------------------
int init() {
   string sType;   
   string sTF;

   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(0,dBuffer0);
   SetIndexLabel(0,"Mkt-Up");
   SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(1,dBuffer1);
   SetIndexLabel(1,"Mkt-Dn");
   SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(2,dBuffer2);
   SetIndexLabel(2,"Sig-Up");
   SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(3,dBuffer3);
   SetIndexLabel(3,"Sig-Dn");
   IndicatorDigits(2);
    
  if (Show.TimeFrame==0) 
    sTF="";
  else
    sTF=TF2Str(Show.TimeFrame);
  IndicatorShortName(INDICATOR_NAME+" "+sTF);
  
  DeleteObjects(INDICATOR_NAME); 
  bStartup=true;
  return(0);
}


//-----------------------------------------------------------------------------
// function: deinit()
// Description: Custom indicator deinitialization function.
//-----------------------------------------------------------------------------
int deinit() {
  DeleteObjects(INDICATOR_NAME);
  return (0);
}


///-----------------------------------------------------------------------------
// function: start()
// Description: Custom indicator iteration function.
//-----------------------------------------------------------------------------
int start() {
  int iNewBars;
  int iCountedBars; 
  int i;
  double dVBHighLine;        // Upper volatility band - Upper Blue line
  double dVBLowLine;         // Lower volatility band - Lower Blue line
  double dMarketBaseLine;    // Market Base Line - Yellow line
  double dRSIPriceLine;      // RSI PriceLine - Green line
  double dTradeSignalLine;   // Trade Signal Line - Red line
  
  // Get unprocessed ticks
  iCountedBars=IndicatorCounted();
  if(iCountedBars < 0) return (-1); 
  if(iCountedBars>0) iCountedBars--;
  iNewBars=Bars-iCountedBars;
  
  for(i=iNewBars; i>=0; i--) {      
    // Get TDI Data
    //dVBHighLine=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
    //                    TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,1,i);       // Upper Blue line
    dMarketBaseLine=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                            TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,2,i);   // Yellow line
    //dVBLowLine=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
    //                   TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,3,i);        // Lower Blue line
    dRSIPriceLine=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                          TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,4,i);     // Green line
    dTradeSignalLine=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                            TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,5,i);  // Red line

    // Clear current display buffer
    dBuffer0[i] = EMPTY_VALUE; 
    dBuffer1[i] = EMPTY_VALUE;
    dBuffer2[i] = EMPTY_VALUE;
    dBuffer3[i] = EMPTY_VALUE;

    // Update current display buffer
    if (Show.MarketBaseLine) {
      if (dRSIPriceLine>dTradeSignalLine && dRSIPriceLine>dMarketBaseLine) 
        dBuffer0[i] = BAR_HEIGHT;
      else if (dRSIPriceLine<dTradeSignalLine && dRSIPriceLine<dMarketBaseLine) 
        dBuffer1[i] = BAR_HEIGHT;
      else if (dRSIPriceLine>dTradeSignalLine) 
        dBuffer2[i] = BAR_HEIGHT;
      else if (dRSIPriceLine<dTradeSignalLine) 
        dBuffer3[i] = BAR_HEIGHT;
      }
    else {   
      if (dRSIPriceLine>dTradeSignalLine) 
        dBuffer0[i] = BAR_HEIGHT;
      else if (dRSIPriceLine<dTradeSignalLine) 
        dBuffer1[i] = BAR_HEIGHT;
      else 
        dBuffer2[i] = BAR_HEIGHT;
    }
    // Show arrows on line crosses
    if (Show.RSIPriceLineArrow || Show.MarketBaseLineArrow) DisplayCrossArrows(i);
    // Check for alerts
    if (i==0) AlertCheck();
  }
  
  // Reset startup flag
  if (bStartup) bStartup=false;
  
  // Bye bye  
  return(0);
}
//+------------------------------------------------------------------+


//-----------------------------------------------------------------------------
// function: TF2Str()
// Description: Convert time-frame to a string
//-----------------------------------------------------------------------------
string TF2Str(int iPeriod) {
  switch(iPeriod) {
    case PERIOD_M1: return("M1");
    case PERIOD_M5: return("M5");
    case PERIOD_M15: return("M15");
    case PERIOD_M30: return("M30");
    case PERIOD_H1: return("H1");
    case PERIOD_H4: return("H4");
    case PERIOD_D1: return("D1");
    case PERIOD_W1: return("W1");
    case PERIOD_MN1: return("MN1");
    default: return("M"+iPeriod);
  }
}


//-----------------------------------------------------------------------------
// function: AlertCheck()
// Description: Check for alert conditions and perform alerts
//-----------------------------------------------------------------------------
void AlertCheck() {
  string sAlertMsg;
  static datetime tCurTime=0;  
  static int iAlertTimer[2];  
  static int iAlertStatus[2];  
  double dMarketBaseLine[2];    // Market Base Line - Yellow line
  double dRSIPriceLine[2];      // RSI PriceLine - Green line
  double dTradeSignalLine[2];   // Trade Signal Line - Red line
  int i,j;
  bool bNewBar=false;
  
  // Abort if indicator just started up
  if (bStartup) {
    tCurTime=Time[0];
    return(0);
  }

  // Check for alerts
  if (Alert.RSIPriceLine || Alert.MarketBaseLine) {     
    
    // Alert if newbar only
    if (Alert.NewBarOnly) {
      // Reset Time
      if(tCurTime!=Time[0]) {
        tCurTime=Time[0];
        bNewBar=true;
      
        // Get TDI Data on new bar, ie bar 1 and 2
        for (i=0; i<2; i++) {
          j=i+1;
          dMarketBaseLine[i]=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                                     TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,2,j);   // Yellow line
          dRSIPriceLine[i]=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                                   TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,4,j);     // Green line
          dTradeSignalLine[i]=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                                      TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,5,j);  // Red line
        } //endfor
      }//endif
    }
    else {
      // Get TDI Data on current bar, ie bar 0 and 1
      for (i=0; i<2; i++) {
        dMarketBaseLine[i]=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                                   TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,2,i);   // Yellow line
        dRSIPriceLine[i]=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                                 TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,4,i);     // Green line
        dTradeSignalLine[i]=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                                    TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,5,i);  // Red line
      } //endfor
    } //end if(Alert.NewBarOnly)  
          
    if (bNewBar || (!Alert.NewBarOnly && iAlertTimer[0]<GetTickCount())) {
      //Alert RSIPriceLine > TradeSignalLine
      if (dRSIPriceLine[0]>dTradeSignalLine[0] && dRSIPriceLine[1]<dTradeSignalLine[1] && Alert.RSIPriceLine) {
          if (!Alert.NewBarOnly) iAlertTimer[0]=GetTickCount()+Alert.Timeout*1000;
          sAlertMsg="TDI Alert - "+Symbol()+" "+TF2Str(Period())+": RSIPriceLine Above TradeSignalLine.";
          if (Alert.Popup) Alert(sAlertMsg);      
          if (Alert.Email) SendMail( sAlertMsg, "MT4 Alert!\n" + TimeToStr(TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);
      } //end if
      
      //Alert RSIPriceLine < TradeSignalLine
      if (dRSIPriceLine[0]<dTradeSignalLine[0] && dRSIPriceLine[1]>dTradeSignalLine[1] && Alert.RSIPriceLine) {
        //PrintD("RSIPriceLine < TradeSignalLine");
        if (!Alert.NewBarOnly) iAlertTimer[0]=GetTickCount()+Alert.Timeout*1000;
        sAlertMsg="TDI Alert - "+Symbol()+" "+TF2Str(Period())+": RSIPriceLine Below TradeSignalLine.";
        if (Alert.Popup) Alert(sAlertMsg);      
        if (Alert.Email) SendMail( sAlertMsg, "MT4 Alert!\n" + TimeToStr(TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);
      } //end if
    } //end if
      
    if (bNewBar || (!Alert.NewBarOnly && iAlertTimer[1]<GetTickCount())) {     
      //Alert if RSIPriceLine > TradeSignalLine && dMarketBaseLine
      if (dRSIPriceLine[0]>dMarketBaseLine[0] && dRSIPriceLine[1]<dMarketBaseLine[1] && Alert.MarketBaseLine) {
        if (dRSIPriceLine[0]>dTradeSignalLine[0] && dRSIPriceLine[1]<dTradeSignalLine[1]) {
          if (!Alert.NewBarOnly) iAlertTimer[1]=GetTickCount()+Alert.Timeout*1000;
          sAlertMsg="TDI Alert - "+Symbol()+" "+TF2Str(Period())+": RSIPriceLine Above TradeSignalLine and MarketBaseLine.";
          if (Alert.Popup) Alert(sAlertMsg);      
          if (Alert.Email) SendMail( sAlertMsg, "MT4 Alert!\n" + TimeToStr(TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);
        } //end if
      } //end if
      
      //Alert if RSIPriceLine < TradeSignalLine && dMarketBaseLine
      if (dRSIPriceLine[0]<dMarketBaseLine[0] && dRSIPriceLine[1]>dMarketBaseLine[1] && Alert.MarketBaseLine) {
        if (dRSIPriceLine[0]<dTradeSignalLine[0] && dRSIPriceLine[1]>dTradeSignalLine[1] ) {         
          if (!Alert.NewBarOnly) iAlertTimer[1]=GetTickCount()+Alert.Timeout*1000;
          sAlertMsg="TDI Alert - "+Symbol()+" "+TF2Str(Period())+": RSIPriceLine Below TradeSignalLine and MarketBaseLine.";
          if (Alert.Popup) Alert(sAlertMsg);      
          if (Alert.Email) SendMail( sAlertMsg, "MT4 Alert!\n" + TimeToStr(TIME_DATE|TIME_SECONDS )+"\n"+sAlertMsg);
        } //end if
      } //end if
    } //end if
 
  }//end if(Alert.RSIPriceLine || Alert.MarketBaseLine)
  
  return(0);
}

//-----------------------------------------------------------------------------
// function: DisplayCrossArrows()
// Description: Show arrows for all RSIPriceLine/TradeSignalLine and
//              RSIPriceLine/MarketBaseLine crosses
//-----------------------------------------------------------------------------
void DisplayCrossArrows(int iCurBar) {
  double dMarketBaseLine[2];    // Market Base Line - Yellow line
  double dRSIPriceLine[2];      // RSI PriceLine - Green line
  double dTradeSignalLine[2];   // Trade Signal Line - Red line
  int i,j;
  double dOffset;
  int iArrowUp,iArrowDn;

  // Get TDI Data on current bar, ie bar 0 and 1
  for (i=0; i<2; i++) {
    j=iCurBar+i;
    dMarketBaseLine[i]=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                               TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,2,j);   // Yellow line
    dRSIPriceLine[i]=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                             TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,4,j);     // Green line
    dTradeSignalLine[i]=iCustom(NULL,Show.TimeFrame,TDI_INDI,TDI.RSIPeriod,TDI.RSIPrice,TDI.VolatilityBand,TDI.RSIPriceLine,
                                TDI.RSIPriceType,TDI.TradeSignalLine,TDI.TradeSignalType,5,j);  // Red line
  } //endfor
  SetArrowType(iArrowUp,iArrowDn);
  if (Show.RSIPriceLineArrow) {
    if (dRSIPriceLine[0]<dTradeSignalLine[0] && dRSIPriceLine[1]>dTradeSignalLine[1]) {  
      dOffset=Point*(110/MarketInfo(Symbol(),MODE_TICKVALUE));
      DrawArrow(false, Time[iCurBar], High[iCurBar]+dOffset,iArrowDn, Crimson);
    }
    else if (dRSIPriceLine[0]>dTradeSignalLine[0] && dRSIPriceLine[1]<dTradeSignalLine[1] ) {
      dOffset=Point*(50/MarketInfo(Symbol(),MODE_TICKVALUE));
      DrawArrow(false, Time[iCurBar], Low[iCurBar]-dOffset,iArrowUp, Green);
    }
  }
  
  if (Show.MarketBaseLineArrow) {
      if(dRSIPriceLine[0]<dTradeSignalLine[0] && dRSIPriceLine[0]<dMarketBaseLine[0] && dRSIPriceLine[1]>dMarketBaseLine[1]){  
      dOffset=Point*(110/MarketInfo(Symbol(),MODE_TICKVALUE));
      DrawArrow(false, Time[iCurBar], High[iCurBar]+dOffset,iArrowDn,Red);
      }
      else if(dRSIPriceLine[0]>dTradeSignalLine[0] && dRSIPriceLine[0]>dMarketBaseLine[0] && dRSIPriceLine[1]<dMarketBaseLine[1]){  
      dOffset=Point*(50/MarketInfo(Symbol(),MODE_TICKVALUE));
      DrawArrow(false, Time[iCurBar], Low[iCurBar]-dOffset,iArrowUp,LimeGreen);
    }
  }
}

//-----------------------------------------------------------------------------
// function: SetArrowStyle()
// Description: Set the style of arrow to draw
//-----------------------------------------------------------------------------
void SetArrowType(int& iArrowUp, int& iArrowDn) {
  switch (Show.ArrowType) {
    case 1:  // Thin
      iArrowDn=226;
      iArrowUp=225;
    break;
    case 2:  // Hollow
      iArrowDn=242;
      iArrowUp=241;
    break;
    case 3:  // Round
      iArrowDn=222;
      iArrowUp=221;
    break;
    default: // Thick
      iArrowDn=234;
      iArrowUp=233;
    break;
  }
}

//-----------------------------------------------------------------------------
// function: DrawArrow()
// Description: Draw an arrow on the chart
//-----------------------------------------------------------------------------
void DrawArrow(bool bClear, datetime dTime, double dPrice=0, int iDir=0, color cArrowColor=Green) {
  string sObjId;
  int iArrow;
  
  sObjId=INDICATOR_NAME+"_Arrow_"+TimeToStr(dTime,TIME_DATE|TIME_MINUTES);  
  if (bClear) {
    ObjectDelete(sObjId);
  }
  else {
    if (ObjectFind(sObjId)<0)
      ObjectCreate(sObjId, OBJ_ARROW, 0, dTime, dPrice);
    ObjectSet(sObjId, OBJPROP_STYLE, STYLE_SOLID);
    ObjectSet(sObjId, OBJPROP_ARROWCODE, iDir);
    ObjectSet(sObjId, OBJPROP_COLOR,cArrowColor);
    ObjectMove(sObjId, 0, dTime, dPrice);
  }
}

//-----------------------------------------------------------------------------
// function: DeleteObjects()
// Description: Erase objects created by the grid
//-----------------------------------------------------------------------------
int DeleteObjects(string sObjId) {
  // Clear chart objects
  for(int i=ObjectsTotal()-1; i>-1; i--)
   if (StringFind(ObjectName(i),sObjId)>=0)  ObjectDelete(ObjectName(i));
   
  return(0);
} // end funcion()


//****************************************************************************
// Debug Functions
//****************************************************************************
void PrintD(string text = "", bool bClear=false){
  static string list[31];                                       // list size == 30
  static int r = 0;                                             // r is the array element to return to the caller
  
  if (bClear==True) r=0;
  if(text == "") return(list[r]);                               // null text received, return the current comment string
  if(r<ArraySize(list)-1)r++;                                   // increment return pointer while list grows
  for(int i=r-1;i>0;i--){list[i]=list[i-1];}list[r]="";         // make room for new item, clear old concatenated string
  list[0]="\n "+TimeToStr(TimeLocal(),TIME_SECONDS)+"  "+text;  // insert new list item
  for(i=0;i<r;i++){list[r]=list[r]+list[i];}                    // recreate the concatenated string
  Comment(list[r]);                                              // return the concatenated string
 }
 


